@@ -378,6 +378,12 @@ class TestDuplicate(TestsExceptionFilter):
             "sqlite",
             'UNIQUE constraint failed: tbl.a, tbl.b')
 
+    def test_sqlite_dupe_primary_key(self):
+        self._run_dupe_constraint_test(
+            "sqlite",
+            "PRIMARY KEY must be unique 'insert into t values(10)'",
+            expected_columns=[])
+
     def test_mysql_mysqldb(self):
         self._run_dupe_constraint_test(
             "mysql",
@@ -491,6 +497,13 @@ class TestDeadlock(TestsExceptionFilter):
             "transaction')"
         )
 
+    def test_mysql_mysqldb_galera_deadlock(self):
+        self._run_deadlock_detect_test(
+            "mysql",
+            "(1205, 'Lock wait timeout exceeded; "
+            "try restarting transaction')"
+        )
+
     def test_mysql_mysqlconnector_deadlock(self):
         self._run_deadlock_detect_test(
             "mysql",
